function slml2img( varargin )
% SLML2IMG Synthesizes a set of multicolor digital images from a collection
% of SLML Level 1.0 Version 1.* instances.
%
% COMMENT: When the method is not deployed the shape of the input arguments is
% slml2img( models, param )
%
% List Of Input Arguments  Descriptions
% -----------------------  ------------
% models                   A cell array of filenames
% param                    A structure holding the function options
%
% The shape of param is described
%
% List Of Parameters        Descriptions
% ------------------        ------------
% targetDirectory           (optional) Directory where the images are going to be saved. Default is current directory.
% prefix                    (optional) Filename prefix for the synthesized images. Default is 'demo'
% numberOfSynthesizedImages (optional) Number of synthesized images. Default is 1.
% compression               (optional) Compression of tiff, i.e. 'none', 'lzw' and 'packbits'
% verbose                   (optional) Print the intermediate steps to screen. Default is true.
% microscope                (optional) Microscope model from which we select a point spread function. Default is 'none'
% sampling.method           (optional) Can be 'disc', 'trimmed' or 'sampled'. Default is trimmed
% sampling.method.density   (optional) An integer. Default is empty.
% protein.cytonuclearflag   (optional) Can 'cyto', 'nucleus' or 'all'. Default is all.
%
% Example
% -------
% instances = { 'model01.xml', 'model02.mat', 'model03.mat' };
% param.targetDirectory = pwd;
% param.prefix = 'demo'
% param.numberOfSynthesizedImages = 100;
% param.compression = 'lzw';
% param.microscope = 'svi';
% param.verbose = true;
%
% >> slml2img( instances, param );

% Author: Ivan E. Cao-Berg (icaoberg@cmu.edu)
% Created: May 8, 2007
% March 7, 2012 R.F.Murphy  Don't create directory to hold 3D images if
%                           generating 2D
% March 8, 2012 I. Cao-Berg Documented the method
% March 9, 2012 I. Cao-Berg Changed disp to fprintf and improved logs
% March 9, 2012 I. Cao-Berg Removed print of progress bar from log file
% March 21, 2012 I. Cao-Berg Changed protein.location to protein.cytonuclearflag
% April 11, 2012 I. Cao-Berg Removed removal of temporary folder
%
% Copyright (C) 2007-2012 Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

% if ~isdeployed
%     if nargin ~= 2
%         error( ['CellOrganizer: Wrong number of input arguments. If slml2img ' ...
%             ' is not deployed, then the number of input arguments must be 2.'] );
%     end
%
% end

%check if files exists and are valid SLML instances
%INPUT ARGUMENTS
param = varargin{2};

%INPUT ARGUMENTS
try
    verbose = param.verbose;
    if ~islogical( verbose )
       verbose = true;
    end
catch
    verbose = true;
end

try
    debug = param.debug;
    if ~islogical( debug )
       debug = false;
    end
catch
    debug = false;
end

%LOGS
try
    if verbose
        disp( 'Checking existence of log folder' );
    end
    
    if ~exist( [ pwd filesep 'log'], 'dir' )
        if verbose
            disp( ['Making log directory ' pwd filesep 'log' ] );
        end
        mkdir( 'log' );
    end
    
    c = clock;
    logfile = '';
    for i=1:1:length(c)
        logfile = ['',logfile,num2str(c(i))]; %#ok<AGROW>
    end
    logfile = [ pwd filesep 'log' filesep logfile, '.log' ];
    
    fileID = fopen( logfile, 'w' );
catch err
    warning( 'CellOrganizer: Failed to create log file. Exiting program.' ) %#ok<*WNTAG>
    if ~isdeployed
        return
    else
        exit
    end
end

filenames = varargin{1};
if verbose
    fprintf( 1, '%s%%', 'Checking the validity of input files: 0');
end

fprintf( fileID, '%s', 'Checking the validity of input files' );
for i=1:1:length(filenames)
    if ~isaFile( filenames{i} )
        if verbose
            fprintf( 1, '\n%s', ['Input argument ' filenames{i} ' is not a file'] );
        end
        
        fprintf( fileID, '\n%s', ['Input argument ' filenames{i} ' is not a file'] );
        fclose( fileID );
        
        error( ['CellOrganizer: Input argument ' i ' is not a file'] );
    else
        n=length(filenames);
        if verbose
            %indicate an object has been successfully added
            fprintf( 1, [ repmat('\b', 1, length(num2str(100*(i-1)/n))+1) '%s'], ...
                repmat(' ', 1, length(num2str(100*(i-1)/n))+1) );
            fprintf( 1, [ repmat('\b', 1, length(num2str(100*(i-1)/n))+1) '%s%%'], num2str(100*i/n) );
            pause(0.1)
        end
    end
end

%parse SLML instances into Matlab structures
if verbose
    fprintf( 1, '\n%s%%', 'Parsing SLML instances: 0' );
end

fprintf( fileID, '%s', 'Parsing SLML instances' );
try
    for j=1:1:length(filenames)
        %if it is a mat file then load directly into memory
        if( isMatFile(filenames{j}) )
            load(filenames{j});
            models{j} = model;
        else
            models{j} = slml2model( filenames{j} );
        end
        
        n=length(filenames);
        if verbose
            %indicate an object has been successfully added
            fprintf( 1, [ repmat('\b', 1, length(num2str(100*(j-1)/n))+1) '%s'], ...
                repmat(' ', 1, length(num2str(100*(j-1)/n))+1) );
            fprintf( 1, [ repmat('\b', 1, length(num2str(100*(j-1)/n))+1) '%s%%'], num2str(100*j/n) );
            pause(0.1)
        end
    end
catch err
    warning( 'CellOrganizer: Unable to parse SLML instances. Exiting program.' );
    fprintf( fileID, '%s\n', 'Unable to parse SLML instances. Exiting progam.' );
    if verbose
        fprintf( 1, '%s\n', 'Unable to parse SLML instances. Exiting progam.' );
    end
    
    fprintf( fileID, '%s\n', 'Closing log file' );
    fclose( fileID );
    if ~isdeployed
        return
    else
        exit
    end
end

%TEMPORARY FOLDER
fprintf( fileID, '\n%s\n', 'Checking existence of temporary folder' );
if verbose
    fprintf( 1, '\n%s\n', 'Checking existence of temporary folder' );
end

if exist([pwd filesep 'temp'], 'dir' )
    fprintf( fileID, '%s\n', ['Temporary folder found in ' pwd filesep 'temp. Removing all temporary files.'] );
    if verbose
        fprintf( 1, '%s\n', ['Temporary folder found in ' pwd filesep 'temp. Removing all temporary files.'] );
    end
    rmdir([pwd filesep 'temp'], 's');
    mkdir([pwd filesep 'temp']);
else
    fprintf( fileID, '%s\n', ['Creating temporary folder in: ' pwd filesep 'temp'] );
    if verbose
        fprintf( 1, '%s\n', ['Creating temporary folder in: ' pwd filesep 'temp'] );
    end
    mkdir([pwd filesep 'temp']);
end

%SETTING ARGUMENTS
try
    targetDirectory = param.targetDirectory;
catch err
    targetDirectory = pwd;
end

fprintf( fileID, '%s\n', ['Setting target directory to: ' targetDirectory ]  );
if verbose
    fprintf( 1, '%s\n', ['Setting target directory to: ' targetDirectory ]  );
end

%setting prefix
try
    prefix = param.prefix;
catch
    prefix = 'demo';
end

fprintf( fileID, '%s\n', ['Setting prefix to: ' prefix ]  );
if verbose
    fprintf( 1, '%s\n', ['Setting prefix to: ' prefix ]  );
end

%setting number of synthesized images
if isdeployed
    numberOfSynthesizedImages = str2double( varargin{3} );
else
    try
        numberOfSynthesizedImages =  param.numberOfSynthesizedImages;
    catch err
        numberOfSynthesizedImages = 1;
    end
end

fprintf( fileID, '%s\n', ['Setting number of synthesized images to: ' num2str(numberOfSynthesizedImages) ]  );
if verbose
    fprintf( 1, '%s\n', ['Setting number of synthesized images to: ' num2str(numberOfSynthesizedImages) ]  );
end

%setting compression
try
    compression = param.compression;
catch err
    compression = 'none';
end

fprintf( fileID, '%s\n', ['Setting compression to: ' compression ] );
if verbose
    fprintf( 1, '%s\n', ['Setting compression to: ' compression ] );
end

%synthesize multicolor images and save them to disk
for i=1:1:numberOfSynthesizedImages
    if verbose
        disp( ['Synthesizing image ' num2str(i) '. Please wait, this might take several minutes.' ] );
    end
    
    fprintf( fileID, '%s\n', ['Synthesizing image ' num2str(i) '. Please wait, this might take several minutes.' ] );
    
    try
        param.fileID = fileID;
        imgs = model2img( models, param );
        
        if isempty( imgs )
            warning('CellOrganizer: Unable to synthesize an image from the given models. Exiting program.');
            fprintf(fileID, '%s\n', 'Unable to synthesize an image from the given models. Exiting progam.');
            if verbose
                disp('Closing log file.')
            end
            
            fprintf( fileID, '%s\n', 'Closing log file' );
            fclose( fileID );
            
            if ~exist([pwd filesep 'temp'])
                rmdir([pwd filesep 'temp'],'s')
            end
            
            if ~isdeployed
                return
            else
                exit
            end
        end
    catch err
        if debug
            disp(err);
            disp(err.message);
            disp(err.stack);
            disp(err.identifier);
        end
        
        warning( 'CellOrganizer: Unable to synthesize an image from the given models. Exiting program.' );
        fprintf( fileID, '%s\n', 'Unable to synthesize an image from the given models. Exiting progam.' );
        if verbose
            disp('Closing log file.')
        end
        
        fprintf( fileID, '%s\n', 'Closing log file' );
        fclose( fileID );
        
        if ~exist([pwd filesep 'temp'])
            rmdir([pwd filesep 'temp'],'s');
        end
        
        if ~isdeployed
            return
        else
            exit
        end
    end
    
    if strcmpi( models{1}.dimensionality, '2d' )
        load( [ pwd filesep 'temp' filesep 'image.mat' ] );
        img2tif( imgs,[ targetDirectory filesep prefix num2str(i) ...
            '.tif'], compression );
        clear img
    else
        outdir = [ targetDirectory filesep prefix filesep 'cell' num2str(i) ];
        if ~exist( outdir , 'dir' )
            mkdir( outdir )
        end
        
        load( [ pwd filesep 'temp' filesep 'image1.mat' ] );
        if verbose
            disp( 'Saving nuclear channel image' );
        end
        
        fprintf( fileID, '%s\n', 'Saving nuclear channel image'  );
        
        img2tif( img, [ outdir filesep 'nucleus.tif'], compression );
        clear image;
        
        load( [ pwd filesep 'temp' filesep 'image2.mat' ] );
        if verbose
            disp( 'Saving cell channel image' );
        end
        
        fprintf( fileID, '%s\n', 'Saving cell channel image'  );
        
        img2tif( img, [ outdir filesep 'cell.tif'], compression );
        clear image;
        
        for j=1:1:length(models)
            if ~strcmpi( models{j}.proteinShape.class, 'centrosome' )
                if verbose
                    disp( ['Saving protein channel image ' models{j}.proteinShape.class ] );
                end
                fprintf( fileID, '%s\n', ['Saving protein channel image ' models{j}.proteinShape.class] );
                
                load( [ pwd filesep 'temp' filesep 'image' num2str(j+2) '.mat' ] )
                img2tif( img, [ outdir filesep models{j}.proteinShape.class '.tif'], compression );
            end
        end
    end
end

if ~debug
 fprintf( fileID, '%s\n', 'Removing temporary folder' );
 if verbose
    fprintf( 1, '%s\n', 'Removing temporary folder' );
    rmdir( 'temp', 's' );   
 end
end

if verbose
    disp('Closing log file.')
end

fprintf( fileID, '%s\n', 'Closing log file' );
fclose( fileID );
