function centrosome = model2centrosome(segdna, segcell, model)
%MODEL2CENTROSOME Generates a centrosome location from a generative model of protein location.

% Author: Jieyue Li
% Edited: Ivan E. Cao-Berg (icaoberg@scs.cmu.edu)
% Edited: Devin Sullivan March, 18, 2012
%   - added up and down sampling 
%   - changed from model.parameters.slice to model.parameters.zloc, change
%   should be backwards compatible
%
%
% Copyright (C) 2012 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


% Dbothfin: cytosolic space, 0 - inside, 1 - outside;
% segdna: segmented nucleus, 0 - outside, 1 - inside;
% segcell: segmented cell,  0 - outside, 1 - inside.
% rand_seed: rand seed for generation.

centrosome = [];

Dbothfin = (logical(~segcell)+logical(segdna));

%%
%Added by DPS 3/18/12
%Downsample Dboth and make sure it is 256x256
Dsizes = size(Dbothfin);
DmaxDim = max(Dsizes(1:2));

% first make sure the x and y dimensions are the same
% if not add a ones cushion (zeros are for the allowable MT space)
if(Dsizes(1)~=Dsizes(2))
    diff = Dsizes(1)-Dsizes(2);
    disp('cell image x and y dimension not equal, adding cushion');
    if(diff>0)
        Dbothfin = [Dbothfin,ones(Dsizes(1),diff,Dsizes(3))];
        segcell = [segcell,zeros(Dsizes(1),diff,Dsizes(3))];
        segdna = [segdna,zeros(Dsizes(1),diff,Dsizes(3))];
    elseif(diff<0)
        Dbothfin = [Dbothfin;ones(-diff,Dsizes(2),Dsizes(3))];
        segcell = [segcell;zeros(-diff,Dsizes(2),Dsizes(3))];
        segdna = [segdna;zeros(-diff,Dsizes(2),Dsizes(3))];
    end
end

%Do the downsampling
if(size(Dbothfin,1)==256)
    disp('No downsampling required, img 256x256xZ');
else
   disp(['Downsampling img:',num2str(size(Dbothfin)),'to 256x256x(Znew)']);
   %segcell
    segcell = imresize(segcell,(256/DmaxDim));
    %need to resize the zdimension
    segcell = tp_stretch3d(segcell,floor(Dsizes(3)*(256/DmaxDim)));
    %segdna
    segdna = imresize(segdna,(256/DmaxDim));
    %need to resize the zdimension
    segdna = tp_stretch3d(segdna,floor(Dsizes(3)*(256/DmaxDim)));
    %Dboth
    Dbothfin = imresize(Dbothfin,(256/DmaxDim));
    %need to resize the zdimension
    Dbothfin = tp_stretch3d(Dbothfin,floor(Dsizes(3)*(256/DmaxDim)));
end

%turn it back into a logical 
%Dbothfin = logical(Dbothfin); % actually not a logical 
%End 3/9/12 addition
%%

if ~isfield( model, 'class' )
   warning( 'CellOrganizer: Model class not present.' );
   return;
end

if ~strcmpi( lower(model.class), 'centrosome' )
   warning( 'CellOrganizer: Model class must be centrosome' );
   return;
end

if ~isfield( model, 'dimensionality' )
   warning( 'CellOrganizer: Model dimensionality not present. Assuming 3D.' );
end

if ~strcmpi( lower(model.dimensionality), '3d' )
  warning( ['CellOrganizer: Model dimensionality not 3D. This version only supports 3D ' ...
      'generative models of centrosomes.'] );
  return;
end

try
  cent_dist = model.parameters.distance;
catch
  warning( 'CellOrganizer: Incomplete model. Parameter distance not present or invalid.' );
  return;
end

try
  p = model.parameters.p;
catch
  warning( 'CellOrganizer: Incomplete model. Parameter p not present or invalid.' );
end

try
    try
      fractionZloc = model.parameters.zloc;
      slice = floor(fractionZloc*size(Dbothfin,3));
    catch
        warning('CellOrganizer: Old model type, using model.parameters.slice instead of model.parameters.zloc');
        slice = model.parameters.slice;
    end
  
catch
  warning( 'CellOrganizer: Incomplete model. Parameter slice not present or invalid.' );
end

try
 R = gamrnd(p(1),p(2));

 [D1] = bwdist(segdna(:,:,slice)>0);
 [D2] = bwdist((~segcell(:,:,slice))>0);
 
 Dbothfin = double(~Dbothfin(:,:,slice));
 Dbothfin(Dbothfin==0) = NaN;
 D1 = D1.*double(Dbothfin); 
 D2 = D2.*double(Dbothfin);
 
 DD = D1./D2;
 DD = DD - R;
  DD = abs(DD);%*4));%removed DPS 3/18/12 at the direction of Jieyue 

 [a,b] = find(DD==nanmin(nanmin(DD)));
 aaa = randperm(length(a));
 
 imgcent_coordinate = [a(aaa(1)),b(aaa(1)),slice];%slice*4];
 centrosome = imgcent_coordinate;
 
%%
%Added DPS 3/18/12
disp('Adjust centrosome location to match original image size');
centrosome = floor(centrosome.*(DmaxDim/256));
%end 3/18/12 upsampling addition
%%
 save temp
catch
 warning('CellOrganizer: Cannot generate centrosomal location');
end


