function [MTimage,MTimagewpsf] = microtubules(Dboth,imgcent_coordinate,microscope,datamat,XYZres)
%
%Devin Sullivan 2/19/12
%This function generates a microtubule from a given model and parameters from a data set (datamat)
%
%Inputs:
%Dboth = cell+nuc;
%   cell = cell shape model(negative, e.g. 0's inside cell)
%   nuc = nuclear shape model (negative e.g. 0's inside nuc)
%imgcent_coordinate = coordinate
%microscope = name of microscope used to generate psf
%datamat = matrix of microtubule parameter values from a given dataset
%XYZres = resolution of the pixels in microns
%Outputs:
%MTimage 
%MTimagewpsf
%

%note, this program assumes that the imgcent_coordinate is taken from the
%cell image and not the nuclear image and allows the cell and nuclear
%images to be potentially different sizes

%need to resize the cell and nuc images (downsample) for speed
%first make sure the image has the same x by y dimensions
if(size(Dboth,1)~=size(Dboth,2))
    diff = size(Dboth,1)-size(Dboth,2);
    disp('cell image x and y dimension not equal, adding cushion');
    if(diff>0)
        Dboth = [Dboth,zeros(size(Dboth,1),diff,size(Dboth,3))];
    elseif(diff<0)
        Dboth = [Dboth;zeros(diff,size(Dboth,2),size(Dboth,3))];
    end
end

%do the downsampling
if(size(Dboth,1)==256)
    disp('No downsampling required, img 256x256xZ');
else
   disp(['Downsampling img:',size(cell),'to 256x256x(Znew)']);
    Dboth = imresize(Dboth,(256/size(Dboth,1)));
    %not sure this is correct
    disp('also adjusting imgcent_coordinate');
    imgcent_coordinate = floor(imgcent_coordinate.*(256/size(Dboth,1)));    
end

if~exist('XYZres','var')
    disp('no resolution specified, using default value from HeLa dataset');
    XYZres = 0.2
end
%if no datamat is provided
if ~exist('datamat','var')
    disp('no data provided for MT parameters, using HeLa parameters');
    load('HeLaresult.mat','finTable');
    datamat = finTable(:,[1,2,4]);
end

%find the mean and covariance for each parameter
meandata = mean(datamat);
covmat = cov(datamat);

%take a random sample from the multivariate gaussian defined by the mean
%and covariance matrix from the data
sample = mvnrnd(meandata,covmat);
n = sample(1);
n = ceil(n);
mu_len = sample(2);
mu_len = ceil(mu_len);
collin_min_number = sample(3);
collin_min_number = ceil(collin_min_number*100)/100;


%sythesize a microtubule distribution 
disp('generating MTimage');
[MTimage,imXYZ,mtXYZ,randlengths] = MT_synth_dps(n,mu_len,collin_min_number,Dboth,imgcent_coordinate,XYZres);
disp('fetching psf');
%get the psf and apply it
psffile = which_psf(microscope);
if(length(psffile)<1)
    disp('no psf, returning original');
else
  MTimagewpsf = psf_blur_hela_mean_dps(MTimage,psffile);    
end

%upsampling image to 1024x1024 for saving 
%all returned images should be 256x256xZ unless unsupported flag is thrown
    disp(['resizing image for display: ' int2str(size(MTimage)),'to 1024x1024x(Znew)']);
    MTimage = imresize(MTimage,(1024/size(MTimage)));
    if(exist(MTimagewpsf,'var'))
        MTimagewpsf = imresize(MTimagewpsf,(1024/size(MTimage)));    
    end
    %shouldn't need to resize these, but just incase you want them to match
    %MTimg/eachother, e.g. if you originally passed a 512x512 cell/nuc. 
    Dboth = imresize(Dboth,4);
    
%%%%%
%optional, uncomment if you want it
%Show the image in a pretty surface plot
%%need to also define these two variables
%segcell = cell shape 
%segdna = nuc shape
%show_3D_image_dps(n,mu_len,colli_min_number,segdna,segcell,psffile,imagepath)
%%%%%

%save the generated images 
save(['outputs/images/new_syn_n_' num2str(n) '_mulen_' num2str(mu_len) '_colli_' num2str(collin_min_number) '.mat'],'MTimage','MTimagewpsf','microscope'); 

