function [spfeat,surfmap,img] = tp_nucimgfeat(imgdir,method,param)
%TP_NUCIMGFEAT Calculates spline features on a nuclear image

% Author: Tao Peng
%
% Copyright (C) 2012 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% March 22, 2012 Added param.downsample so that user can select a 
%                downsampling scale. The default is [5 5 1] which is 
%                the one in the original generative model paper
% March 23, 2012 Fixed a bug when getting cellnum from imgdir
% March 26, 2012 Added image rotation after preprocessing and well as 
%                removal of blank slices on top and bottom of preprocessed 
%                image
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

temporaryFilesFolder = [ pwd filesep 'temp' filesep 'splines' ];
if ~exist( temporaryFilesFolder, 'dir' )
    mkdir( temporaryFilesFolder );
end

if nargin == 2
   param = [];
end

try
  verbose = param.verbose;
catch
  verbose = false;
end

try
   downsample = param.downsample;
catch
   downsample = [5 5 1];
end

%icaoberg
delimiters = find( imgdir=='/' );
cellnum = str2num( imgdir(delimiters(end)+5:end) );

if ~exist([temporaryFilesFolder filesep 'cell' num2str(cellnum) '.mat'], 'file' )
 [segdna,segcell] = seg_cell_and_nucleus( imgdir, downsample );

 %icaoberg - march 26, 2012 added image rotation
 img = double(segdna);

 try
   % image rotation
   nnzpixel = squeeze(sum(sum(img)));
   [maxfluo,supid] = max(nnzpixel);
   supshape = img(:,:,supid);
   theta = ml_majorangle(supshape)*180/pi;
   for k = find(nnzpixel,1):find(nnzpixel,1,'last')
       img(:,:,k) = ml_rotate(img(:,:,k),-theta);
   end
 catch error
   spfeat = [];
   disp( ['Could not rotate image ' imgdir ]);
   return
 end
 
 % spline surface fitting
 img = tp_imtight(img);
 switch method
     case 'medsurf'
         [medplane,height,mask] = tp_imaxisplane(img);
         [medfeat,heifeat] = tp_medplanefeat(medplane,height,mask);
         spfeat = struct('medsurf',medfeat,'height',heifeat);
     case 'cylsurf'
         surfmap = tp_surfmap(img,2*pi/360);
         spfeat = tp_spcylfeat(surfmap);
 end

 save([temporaryFilesFolder filesep 'cell' num2str(cellnum) '.mat'], ...
     'spfeat', 'surfmap', 'img');
 return
else
   if verbose
     fprintf(1,'%s\n',['Spline features found for image ' num2str(cellnum) '.']);      
   end
   load([temporaryFilesFolder filesep 'cell' num2str(cellnum) '.mat']);
   return
end
end
