function [obj,protimg] = tp_gengaussobjs(varargin)
% Generate a set of gaussian objects from the model
% Graphical Model: Y <-- X --> Z
%This method can be run in two modes. The mode is determined by the number
%of input arguments.
%
%If the variables protimg and newpos ARE NOT specified the function will
%generate gaussian discs saved as a cell array within the output 'obj'.
%These discs are sampled from the model mean and the generated cov. below.
%They are cut off at 2 standard deviations. protimgsampled will be returned
%[]
%
%If the variables protimg and newpos ARE specified, the function will
%return an image containing the sampled gaussians in the 'protimgsampled'
%image. 'obj' and img will be returned []

% Author: Tao Peng
% March 9, 2012 I. Cao-Berg Changed report of synthesized objects
%
% Copyright (C) 2008-2011 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%Author: Tao Peng
%Edited: Devin Sullivan March,2012
%   - added sampling method
%   - added direct object addition
%   - added trimmed method
%Edited: I. Cao-Berg March,2012
%   - added varargin structure
%Edited: Devin Sullivan 3/14/12
%   - added user specified sampling density as varargin{6}
%   - fixed trimmed contrast stretching
%   - fixed trimmed point mass
%   - fixed trimmed object 'add' instead of 'replace' 

i = 0;

if(nargin<2)
    error('not enough input arguments for gengaussobjs');
end

model = varargin{1};
n = varargin{2};

% if(nargin<4)
%     %disp('either protimg or newpos were not specified, disk method only, protimg sampled will be []');
%     protimgsampled = [];
%     obj = cell(n,1);
% else
protimg = varargin{3};
newpos = varargin{4};
objectmethod = varargin{5};
param.samplingDensity = varargin{6}; 
if ~(strcmpi(objectmethod,'sampled')|| ...
        strcmpi(objectmethod,'trimmed')|| ...
        strcmpi(objectmethod,'disc'))
    warning('CellOgranizer: Unrecognized object fill method, defaulting to trimmed');
    objectmethod = 'trimmed';
end
obj = [];
img = [];

fprintf( 1, '%s%%', '0');
while i < n
    % Generate covariance matrices
    x = ml_rnd(model.x);
    %x = 2;
    model.y_x.mu = model.y_x.a1*(1-exp(-model.y_x.b1*x));
    model.z_x.mu = model.z_x.a1*(1-exp(-model.z_x.b1*x));
    model.y_x.sigma = model.y_x.a2*(1-exp(-model.y_x.b2*x));
    model.z_x.sigma = model.z_x.a2*(1-exp(-model.z_x.b2*x));
    y = ml_rnd(model.y_x);
    z = ml_rnd(model.z_x);
    
    if y>x || z>x || z>y || x<=0 || y<=0 || z<=0
        continue
    end
    i = i+1;
    %     newSigma = diag([x y z].^2)/25;
    newSigma = diag([x y z].^2);
    
    %if any of the sigmas are less than 1 pixel, set them to 1.
    %this prevents point masses
    if(sum(newSigma(newSigma>0)<1)>0)
        newSigma(newSigma~=0&newSigma<1) = 1;
    end
    
    % Random rotation
    t = rand * pi;
    Rx = [1 0 0;0 cos(t) -sin(t);0 sin(t) cos(t)];
    t = rand * pi;
    Ry = [cos(t) 0 sin(t);0 1 0;-sin(t) 0 cos(t)];
    t = rand * pi;
    Rz = [cos(t) -sin(t) 0;sin(t) cos(t) 0;0 0 1];
    R = Rz*Ry*Rx;
    newCov = R*newSigma*R';
    mumat = newpos(i,:);
    

    if (strcmpi(objectmethod,'sampled'))
        %sampling mode: fill protimg sampled with samples from the
        %multivariate gaussians
%         protimg = gauss_sampler(mumat,newCov,uint8(protimg),param);
        %%
        %attempt to make more efficient 
        numsamples{i} = ceil(param.samplingDensity*sum(abs(newCov(:))));%c*inten_coeff;
        %randomly sample the multivariate gaussian
        %mu is mean, sigma is covariance 
        %this should return a multivariate gaussian approximating 
        Rnums = mvnrnd(mumat,newCov,numsamples{i});
        Rnumrnd{i} = ceil(Rnums);
        %end 3/19/12 edit
        %%
    elseif(strcmpi(objectmethod,'trimmed'))
        obj = ml_gaussobj(newCov);
        inten_coeff = size(obj,1);
        obj(:,end) = obj(:,end)*inten_coeff;
%         if(max(obj(:,end))>1)
%             keyboard
%         end
        protimg = ml_imaddobj2(protimg,obj,...
            struct('method','add','pos',mumat));
        %       img{i} = ml_obj2img(obj{i},[],{'3d','ct'});
    elseif(strcmpi(objectmethod,'disc'))
        % Generate Gaussian object
        obj{i} = ml_gaussobj(newCov);
        % force all intensities to 255
         obj{i}(:,end) = 255;
        %here 'replace' is used since all objs are max brightness already
%          protimg = ml_imaddobj2(protimg,obj{i},...
%              struct('method','replace','pos',mumat));
        %       img{i} = ml_obj2img(obj{i},[],{'3d','ct'});
    else%no recognized sampling option, using 'disc' as default
       % Generate Gaussian object
        obj{i} = ml_gaussobj(newCov);
        % force all intensities to 255
         obj{i}(:,end) = 255;
    end
    %indicate an object has been successfully added
    fprintf( 1, [ repmat('\b', 1, length(num2str(100*(i-1)/n))+1) '%s'], ...
             repmat(' ', 1, length(num2str(100*(i-1)/n))+1) );
    fprintf( 1, [ repmat('\b', 1, length(num2str(100*(i-1)/n))+1) '%s%%'], num2str(100*i/n) );
    %fprintf( 1, '%s', '.');
end
fprintf( 1, '%s\n', '' );
%do the contrast stretching before converting to uint8
if(strcmpi(objectmethod,'disc'))
%     if(strcmpi(objectmethod,'disc'))
%         obj(:,4) = 255;
%     end
        protimg = ml_imaddobj2(protimg,obj,...
            struct('method','replace','pos',newpos));
elseif(strcmpi(objectmethod,'trimmed'))
        protimg = ml_imaddobj2(protimg,obj,...
            struct('method','add','pos',newpos));
elseif(strcmpi(objectmethod,'sampled'))
    %add in sampled image
    param.numsamplescell = numsamples;
    param.Rnumrndcell = Rnumrnd;
    protimg = gauss_sampler(mumat,newCov,uint8(protimg),param);
else
    warning('Unrecognized sampling method, defaulting to disc');
        protimg = ml_imaddobj2(protimg,obj,...
            struct('method','replace','pos',newpos));
end
protimgtmp = ml_bcimg(protimg,[],[0 255]);
protimg = uint8(protimgtmp);

