function model = img2model( dimensionality, varargin )
%IMG2MODEL Trains a generative model of protein subcellular location from a
%collection of microscope images.

% Author: Ivan E. Cao-Berg
% Copyright (C) 2007-2012 Murphy Lab
% Carnegie Mellon University
%
% ?? ??, 2011 I. Cao-Berg Added 3D model training functionality
% March 23, 2012 I. Cao-Berg Added the creation/deletion of temporary folder
% March 28, 2012 I. Cao-Berg Added a control structure under which if one or more of the 
%                image folders are nonexistent or they do not contain images,
%                the method exits
% March 28, 2012 I. Cao-Berg Added verification of input arguments when training a 2D
%                            generative model
% March 28, 2012 I. Cao-Berg Added verification of input arguments when training a 3D
%                            generative model
% April 10, 2012 I. Cao-Berg Added debug flag to the method. If flag is true, temporary
%                            files will not be deleted
% April 11, 2012 I. Cao-Berg Added verbose flag to the method
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu/ or
% send email to murphy@cmu.edu

% Get collection of images
nuclearModelImages = [];
cellModelImages = [];
proteinModelImages = [];

model = [];
switch lower(dimensionality)
    case '2d'
        %icaoberg march 28, 2012
        %check number of input arguments. if there are not 5 input
        %arguments, then the method returns an empty model
        if nargin ~= 5
           warning('CellOrganizer: Wrong number of input arguments.');
           return;
        end

        %check the image directories. if they don't exist or they are
        %empty, the method returns an empty model
        nuclearModelImagesDirectory = varargin{1};
        if ~isa( nuclearModelImagesDirectory, 'char' )
           warning('CellOrganizer: DNA images directory does not exist.');
           return;
        end

        cellModelImagesDirectory = varargin{2};
        if ~isa( cellModelImagesDirectory, 'char' )
           warning('CellOrganizer: Cell images directory does not exist.');
           return;
        end
 
        proteinModelImagesDirectory = varargin{3};
        if ~isa( proteinModelImagesDirectory, 'char' )
           warning('CellOrganizer: Protein images directory does not exist.');
           return;
        end
    
        nuclearModelImages = getImages( ...
            nuclearModelImagesDirectory );
        cellModelImages = getImages( ...
            cellModelImagesDirectory );
        proteinModelImages = getImages( ...
            proteinModelImagesDirectory );
        
        if isempty( nuclearModelImages ) | ...
           isempty( cellModelImages ) | ...
           isempty( proteinModelImages )
           warning(['CellOrganizer: One or more directories are nonexistent ' ...
             'or they do not contain images.']);
          return
        end

        %check that the input parameter is a structure
        param = varargin{4};
        if ~isa( param, 'struct' )
           warning('CellOrganizer: Input argument parameter must be a struct.');
           return;
        end

        %DEBUG MODE!
        try
            debug = param.debug;
            if ~isa( debug, 'logical' )
                debug = false;
            end
        catch
            debug = false;
        end

        try
           verbose = param.verbose;
           if ~isa( debug, 'logical' )
               verbose = false;
           end
        catch
           debug = false;
        end

        %check existence of temporary folder and make on if it doesn't exist
        if ~exist([pwd filesep 'temp' filesep 'preprocessed'])
          mkdir([pwd filesep 'temp' filesep 'preprocessed']);
        end
        
        % train generative model of protein subcellular location
        param.disp = 'false';
        model = ml_traingenmodel2D( ...
            nuclearModelImages, ...
            cellModelImages, ...
            proteinModelImages, [], param );
    case '3d'
        %icaoberg march 28, 2012
        %check the existence of the image directory
        imgDirectory = varargin{1};
        if ~exist( imgDirectory )
           warning('CellOrganizer: Image directory does not exist.');
           return
        end

        param = varargin{2};
        if ~isa( param , 'struct' )
            warning('CellOrganizer: Input parameter list must be a structure.');
            return
        end

        try
           debug = param.debug;
           if ~isa( debug, 'logical' )
               debug = false;
           end
        catch
           debug = false;
        end

        try
           verbose = param.verbose;
           if ~isa( debug, 'logical' )
               verbose = false;
           end
        catch
           debug = false;
        end

        %check existence of temporary folder and make on if it doesn't exist
        if ~exist( 'temp', 'dir' )
            mkdir( 'temp' );
        end
        
        %try
            model.dimensionality = '3D';
            if verbose
              clc; fprintf( 1, '%s\n', 'Training nuclear shape model' );
            end
            model.nuclearShapeModel = train_nuc_shape_model( ...
                imgDirectory );

            if verbose
              clc; fprintf( 1, '%s\n', 'Training cell shape model' );
            end
            model.cellShapeModel = train_cell_shape_model2( ...
                imgDirectory );

            save( 'model.mat', 'model' );
            return

            if verbose
              clc; fprintf( 1, '%s\n', 'Training protein model' );
            end
            model.proteinModel = train_protein_model( ...
               imgDirectory );
        %catch
        %   warning('CellOrganizer: Unable to train 3D generative model.');
        %   if ~debug
        %      rmdir( 'temp', 's' );
        %   end
        %   return
        %end
    otherwise
        error(['CellOrganizer: Unknown dimensionality ' ...
            dimensionality]);
end

if ~debug
   rmdir( 'temp', 's' );
else
   if verbose
       fprintf(1, '%s\n', ...
         'CellOrganizer: Debug mode is on, temporary files will not be deleted.');
   end
end

end%img2model
