function imgs = model2img( models, param )
% MODEL2IMG Synthesizes a multicolor image from a set of SLML Level 1.0
% Version 1.* instances. Saves the results to an intermedia folder and returns
% the synthesized image.
%
% List Of Input Arguments Descriptions
% ----------------------- ------------
% models                  A cell array of valid and parsed SLML instances.
% param                   A structure holding the function options
%
% The shape of param is described
%
% List Of Parameters        Descriptions
% ------------------        ------------
% targetDirectory           (optional) Directory where the images are going to be saved. Default is current directory.
% prefix                    (optional) Filename prefix for the synthesized images. Default is 'demo'
% numberOfSynthesizedImages (optional) Number of synthesized images. Default is 1.
% compression               (optional) Compression of tiff, i.e. 'none', 'lzw' and 'packbits'
% verbose                   (optional) Print the intermediate steps to screen. Default is true.
% microscope                (optional) Microscope model from which we select a point spread function. Default is 'none'
% sampling.method           (optional) Can be 'disc', 'trimmed' or 'sampled'. Default is trimmed
% sampling.density          (optional) An integer. Default is empty.
% protein.cytonuclearflag   (optional) Can 'cyto', 'nucleus' or 'all'. Default is all.
%
% Example
% -------
% instances = { 'model' };
% param.targetDirectory = pwd;
% param.prefix = 'demo'
% param.numberOfSynthesizedImages = 10;
% param.compression = 'lzw';
% param.microscope = 'svi';
% param.verbose = true;
%
% >> slml2img( instances, param );

% Author: Ivan E. Cao-Berg (icaoberg@cmu.edu)
% March 7, 2012 R. F. Murphy Change scaling to 0-255
% March 8, 2012 I. Cao-Berg Documented method
% March 9, 2012 I. Cao-Berg Commented out sampling density,
%                           changed disp to fprintf and improved logs
% March 14, 2012 I. Cao-Berg Added protein.location parameter
% March 14, 2012 I. Cao-Berg Added contrast stretching for every pattern
% March 21, 2012 I. Cao-Berg Changed protein.location to protein.cytonuclear flag
% April 11, 2012 I. Cao-Berg Parameter structure is set to empty by default
% when the number of input arguments is one. Added try/catch clause to
% determine verbose flag. Default is false.
% April 12, 2012 R.F. Murphy Fix calculation of nucleardist and celldist;
% fix name of microtubule model
%
% Copyright (C) 2008-2012  Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

if nargin > 2
    error( 'CellOrganizer: Wrong number of input arguments.' );
end

if nargin == 1
    param = [];
end

imgs = {};

try
    fileID = param.fileID;
catch err
    fileID = [];
end

try
    sampling = param.sampling.method;
catch err
    param.sampling.method = 'disc';
end

% try
%     N = param.sampling.N;
% catch
%     param.sampling.N = [];
% end
N = [];

try
    logger = param.logger;
catch err
    logger = false;
end

try
    verbose = param.verbose;
catch err
    verbose = true;
end

%checking the validity of the input models

%initialize parameters
if isempty( models )
    error( 'CellOrganizer: Input argument models cannot be empty' );
else
    if ~isempty( fileID )
        fprintf( fileID, '%s\n', ['Setting model dimensionality to ' models{1}.dimensionality] );
    end
    
    if verbose
        fprintf( 1, '%s\n', ['Setting model dimensionality to ' models{1}.dimensionality] );
    end
    
    try
        dimensionality = models{1}.dimensionality;
    catch err
        warning('CellOrganizer: Unable to set model dimensionality');
        return
    end
end

if ~isempty( fileID )
    fprintf( fileID, '%s\n', 'Checking all models have the same dimensionality' );
end
if verbose
    fprintf( 1, '%s\n', 'Checking all models have the same dimensionality' );
end

if ~exist( [ pwd filesep 'temp'], 'dir' )
    mkdir( [pwd filesep 'temp'] );
end

for i=1:1:length(models)
    if ~strcmpi( lower(models{i}.dimensionality), lower(dimensionality) )
        warning('CellOrganizer: Models have different dimensionality. Unable to synthesize image.');
        
        if ~isempty( fileID )
            fprintf( fileID, '%s\n', 'CellOrganizer: Models have different dimensionality. Unable to synthesize image.' );
            fprintf( fileID, '%s\n', 'Closing log file' );
            fclose( fileID );
        end
        
        return;
    end
end

switch lower(dimensionality)
    case '2d'
        if ~exist( 'param', 'var' )
            param  = [];
        end
        
        imgs = [];
        param = ml_initparam(param, ...
            struct('imageSize',[1024 1024],'gentex',0,'loc','all'));
        [nucEdge,cellEdge] = model2framework( models{1}, param );
        %post-process each channel
        se=strel('disk',4,4);
        cellimage = imdilate(cellEdge,se);
        if ~exist('nuctex','var')
            nucimage = imdilate(nucEdge,se);
        else
            nucimage = nucEdge;
        end
        
        for j=1:1:length(models)
            protimage{j} = ...
                ml_genprotimg2D( models{j}.proteinModel, ...
                nucEdge, cellEdge, param );
        end
        
        %post-process each channel
        se=strel('disk',4,4);
        cellimage = imdilate(cellEdge,se);
        if ~exist('nuctex','var')
            nucimage = imdilate(nucEdge,se);
        else
            nucimage = nucEdge;
        end
        
        %add channels into a single image
        imgs(:,:,1) = ml_bcimg(double(nucimage),[],[0 255]);
        imgs(:,:,2) = ml_bcimg(double(cellimage),[],[0 255]);
        for j=1:1:length(protimage)
            imgs(:,:,2+j) = ml_bcimg(double(protimage{j}),[],[0 255]);
        end
        save( ['temp' filesep 'image.mat'], 'imgs' );
    case '3d'
        vesicle = {};
        for j=1:1:length(models)
            if strcmpi( models{i}.proteinShape.type, 'vesicle' )
                vesicle{length(vesicle)+1} = j;
            end
        end
        
        if ~isempty( vesicle )
            fprintf( 1, '%s\n', ['At least one vesicle model was found, looking for a cell shape ' ...
                ' and a nuclear shape model.']);
            if isfield( models{1}, 'nuclearShapeModel' ) && isfield( models{1}, 'cellShapeModel' )
                fprintf( 1, '%s\n', 'A nuclear shape and cell shape was found in the first model.');
            end
        end
        
        microtubules = {};
        for j=1:1:length(models)
            if strcmpi( models{j}.proteinShape.type, 'network' ) & ...
                    strcmpi( models{j}.proteinShape.class, 'microtubule' )
                microtubules{length(microtubules)+1} = j;
            end
        end
        
        if ~isempty( microtubules )
            fprintf( 1, '%s\n', ['At least one microtubule model was found, ' ...
                'looking for a centrosome model.'] );
            centrosome = {};
            for j=1:1:length(models)
                if strcmpi( models{j}.proteinShape.type, 'centrosome' ) & ...
                        strcmpi( models{j}.proteinShape.class, 'centrosome' )
                    centrosome{length(centrosome)+1} = j;
                end
            end
        end
        
        if ~isempty( microtubules ) & isempty( centrosome )
            if ~isempty( fileID )
                fprintf( fileID, '%s\n', ['At least one microtubule model was found ' ...
                    'in your list but no centrosome models were found.'] );
                fprintf( fileID, '%s\n', 'Closing log file' );
                fclose( fileID );
            end
            
            error( ['CellOrganizer: ' num2str(length(microtubules)) ' microtubule models where found ' ...
                'in your list but no centrosome models found.'] ) %uncool
        else
            if exist( 'centrosome', 'var' )
                fprintf( 1, '%s\n', ['Centrosome models found:' num2str(length(centrosome))] );
            end
            
            fprintf( 1, '%s\n', ['Generating cell framework'] );
            [param.nucleus,param.cell]=model2framework( models{1} );
            
            fprintf( 1, '%s\n', ['Filling cell and nuclear images'] );
            param.nucleusfilled=imfill(param.nucleus>0,'holes');
            param.cellfilled=imfill(param.cell>0,'holes');
            
            fprintf( 1, '%s\n', ['Computing Euclidean distance transform to cell and nuclear edges'] );
            param.celldist = bwdist(bwperim(param.cell));
            param.nucleardist = bwdist(bwperim(param.nucleus));
            param.nucleardist(param.nucleusfilled==1) = -param.nucleardist(param.nucleusfilled==1);
            
            if exist( 'centrosome', 'var' ) & ~isempty( centrosome )
                fprintf( 1, '%s\n', ['Generating centrosome.'] );
                param.centrosome = model2instance( models{centrosome{1}}.proteinShape, param );
            end
            
            if verbose
                fprintf( 1, '%s\n', ['Saving temporary cell shape model instance'] );
            end
            img = param.nucleus;
            save( 'temp/image1.mat', 'img' );
            imgs{1} = img;
            clear img;
            
            img = param.cell;
            save( 'temp/image2.mat', 'img' );
            imgs{2} = img;
            clear img;
            
            clear blank;
            clear box;
            clear instance;
            clear nuclei;
            
            for j=1:1:length(models)
                if ~strcmpi( models{j}.proteinShape.class, 'centrosome' )
                    if verbose
                        tic
                        disp( ['Generating protein pattern model instance ' num2str(j) ' from ' models{j}.proteinShape.class ' pattern'] );
                    end
                    
                    img = model2instance( models{j}.proteinShape, param );
                    %img = ml_bcimg(double(img),[],[]);
                    
                    if verbose
                        disp( ['Saving temporary protein model instance ' num2str(j+2) ] );
                    end
                    save( ['temp' filesep 'image' num2str(j+2) '.mat'], 'img' );
                    imgs{length(imgs)+1} = img;
                    clear img;
                end
            end
        end
    otherwise
        error( 'CellOrganizer: Unknown dimensionality or dimensionality argument missing.' );
end
end%model2img
