function object = model2instance( model, param )
% MODEL2INSTANCE Generate a single instance from a valid SLML model. 
% Model is a valid protein pattern model. Depending on the model type, 
% the structure of param changes.

% Author: Ivan E. Cao-Berg (icaoberg@scs.cmu.edu)
% March 8, 2012 Added location option to param structure and set a default value to 'all'
% March 21, 2012 Changed protein.location parameter to protein.cytonuclearflag
%
% Copyright (C) 2011-2012 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

object = [];

if nargin > 2
    error('CellOrganizer: Wrong number of input arguments.');
end

if length(model) > 1
    model = model{1};
end


try
    dimensionality = model.dimensionality;
catch
    warning( 'CellOrganizer: Unable to set dimensionality' );
    return
end

try
    nucleus = param.nucleus;
catch
    warning('CellOrganizer: Parameter nuclear shape instance not set.')
    return
end

try
    cellMembrane = param.cell;
catch
    warning('CellOrganizer: Parameter cell shape instance not set.')
    return
end

try
    modelClass = model.class;
catch
    warning('CellOrganizer: Parameter model class not set.')
    return
end

try
    modelType = model.type;
catch
    warning('CellOrganizer: Parameter model type not set.');
    return
end

try
    microscope = param.microscope;
catch
    microscope = 'none';
end

switch lower(dimensionality)
    case '2d'
        switch lower(modelType)
            case 'vesicle'
                param = ml_initparam(param, ...
                        struct('imageSize',[1024 1024],'gentex',0,'loc','all'));
                object = ml_genprotimg( model, nucleus, cellMembrane, param );
            otherwise
                warning('CellOrganizer: Unknown or unsupported model type.');
                object = [];
                return
        end
    case '3d'
        switch lower(modelType)
            case 'vesicle'
                  %tries to retrieve sampling parameters which if they exist, must already be valid
                  try
                     sampling = param.sampling.method;
                  catch
                     param.sampling.method = 0;
                  end
              
                  try
                     N = param.sampling.N;
                  catch
                     param.sampling.N = [];
                  end                        

                  try
                     location = param.sampling.location;
                  catch
                     param.sampling.location = 'all';
                  end
  
                  object = tp_genprotimage( model, param );

                  psf = micro2psf( microscope );

                  if isempty( psf )
                      return
                  else                        
                      object = psf_blur_hela_mean_dps( object, psf );
                  end
            case 'centrosome'
                  object = model2centrosome(nucleus, cellMembrane, model);
            case 'network'
                  try
                      centrosome = param.centrosome;
                  catch
                      warning(['CellOrganizer: A centrosome instance is needed to ' ...
                          'synthesize a cytoskeletal instance. Remember to initialize param.centrosome.'])
                      object = [];
                      return;
                  end

                  object  = model2microtubules( cellMembrane, nucleus, centrosome, model, param );
                  psf = micro2psf( microscope );
                  if isempty( psf )
                      return
                  else
                      object = psf_blur_hela_mean_dps( object, psf );
                  end
            otherwise
                warning('CellOrganizer: Unknown or unsupported model type.');
                object = [];
                return
        end
    otherwise
        warning('CellOrganizer: Unknown or unsupported dimensionality.')
        object = [];
        return
end



