function [segdna,segcell] = seg_cell_and_nucleus( imgdir, downsample, display, param )

% Author: Robert F. Murphy (murphy@cmu.edu)
%
% Copyright (C) 2012 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

% March 21, 2012 R.F. Murphy Fix downsampling; add display param;
%                            Only keep biggest object in each slice
% March 22, 2012 R.F. Murphy & Devin Sullivan Find top and bottom slice
%                            separately for cell and nucleus
% March 23, 2012 I. Cao-Berg Fixed a bug when getting cellnum from imgdir
% March 25, 2012 I. Cao-Berg The method itself checks whether a temporary file exists
% March 27, 2012 I. Cao-Berg Removed top and bottom blank slices from DNA and cell images
% April 10, 2012 I. Cao-Berg Added parameter structure to hold debug and verbose flags

temporaryFolder = [ pwd filesep 'temp' filesep 'preprocessing' ];
if ~exist( temporaryFolder, 'dir' )
   mkdir( temporaryFolder );
end

segdna = [];
segcell = [];
if nargin == 3
  param = [];
  verbose = false;
  debug = false;
elseif nargin > 4
  warning('CellOrganizer: Wrong number of input arguments.');
  return
else
  try
     verbose = param.verbose;
     if ~islogical( verbose )
        verbose = false;
     end
  catch
     verbose = false;
  end

  try
     debug = param.debug;
     if ~islogical( debug )
        debug = false;
     end
  catch
     debug = false;
  end
end

if ~exist('display','var')
   display = false;
end

%icaoberg fixed bug
delimiters = find( imgdir=='/' );
cellnum = str2num( imgdir(delimiters(end)+5:end) );

%icaoberg march 26, 2012
if exist( [ temporaryFolder ...
   filesep 'cell' num2str(cellnum) '.mat' ] )
   load( [ temporaryFolder ...
   filesep 'cell' num2str(cellnum) '.mat' ] );
   return
end

%protim3 = ml_loadimage([imgdir '/prot/'],'tif');
dnaim3 = ml_loadimage([imgdir '/dna/'],'tif');
cellim3 = ml_loadimage([imgdir '/cell/'],'tif');
mask = ml_loadimage([imgdir '/crop/'],'tif');

%protim3 = tz_maskimg_3d(protim3,mask);
dnaim3 = tz_maskimg_3d(dnaim3,mask);
cellim3 = tz_maskimg_3d(cellim3,mask);
clear mask

% Load PSFs
infdnaPSF = imfinfo( '3DHeLa_DNA_PSF.tif');
infcellPSF = imfinfo( '3DHeLa_Cell_PSF.tif');

dnaPSF = zeros( infdnaPSF(1).Height, infdnaPSF(1).Width, length(infdnaPSF));
cellPSF = zeros( infcellPSF(1).Height, infcellPSF(1).Width, length(infcellPSF));

for I=1:length(infdnaPSF)
    dnaPSF(:,:,I)=imread('3DHeLa_DNA_PSF.tif',I);
end

for I=1:length(infcellPSF)
    cellPSF(:,:,I)=imread('3DHeLa_Cell_PSF.tif',I);
end

dnaPSF = dnaPSF.^2; % Approximate confocal PSF
cellPSF = cellPSF.^2;

% downsample PDFs and images as specified by user
if any(round(downsample)~=downsample) warning('Downsample vector rounded to integers'), end
downsample=round(downsample);

cellfile=['cell_image' num2str(cellnum) '.mat'];
if exist(cellfile,'file')
    load(cellfile)
else
    fprintf(1,'%s\n','Downsampling cell PSF and image'); tic;
    cellPSF = ml_downsize(cellPSF,downsample,'average');
    resizedcell = ml_downsize(cellim3,downsample,'average'); toc
    clear cellim3
    fprintf(1,'%s\n','Deconvolving cell image'); tic;
    [cell_image,cellPSF2] = deconvblind(resizedcell,cellPSF); toc
%    save(cellfile,'cell_image','cellPSF2');
end

dnafile=['dna_image' num2str(cellnum) '.mat'];
if exist(dnafile,'file')
    load(dnafile)
else
    fprintf(1,'%s\n','Downsampling DNA PSF and image'); tic;
    dnaPSF = ml_downsize(dnaPSF,downsample,'average');
    resizeddna = ml_downsize(dnaim3,downsample,'average'); toc
    clear dnaim3
    fprintf(1,'%s\n','Deconvolving DNA image'); tic;
    [dna_image,dnaPSF2] = deconvblind(resizeddna,dnaPSF); toc
%    save(dnafile,'dna_image','dnaPSF2');
end

fprintf(1,'%s\n','Segmenting cell image'); tic;
sumslices = cumsum(squeeze(sum(sum(cell_image))));
sumslices = sumslices/sumslices(end);
bot_slice=find(sumslices>0.02);
bot_slice=bot_slice(1);
top_slice=find(sumslices<0.98);
top_slice=top_slice(end);
fprintf(1,'Total slices=%i, Bottom slice=%i, Top slice=%i\n',length(sumslices),bot_slice,top_slice);
segcell = active3Dsegment(cell_image,bot_slice,top_slice,display);
clear cell_image
toc

fprintf(1,'%s\n','Segmenting DNA image'); tic;
sumslices = cumsum(squeeze(sum(sum(dna_image))));
sumslices = sumslices/sumslices(end);
dbot_slice=find(sumslices>0.05);
dbot_slice=dbot_slice(1);
dtop_slice=find(sumslices<0.95);
dtop_slice=dtop_slice(end);
dbot_slice=max(dbot_slice,bot_slice);
dtop_slice=min(dtop_slice,top_slice);
fprintf(1,'Total slices=%i, Bottom slice=%i, Top slice=%i\n',length(sumslices),dbot_slice,dtop_slice);
segdna = active3Dsegment(dna_image,dbot_slice,dtop_slice,display);
clear dna_image
toc

%icaoberg march 27, 2012
segcell = segcell(:,:,bot_slice:top_slice);
segdna = segdna(:,:,bot_slice:top_slice);

% make sure that no part of the nucleus is outside the cell
segdna = and(segdna,segcell);

if display
    pause
    figure;
    for i=1:size(segdna,3)
        RGB(:,:,1)=uint8(segdna(:,:,i)*255);
        RGB(:,:,2)=uint8(segcell(:,:,i)*255);
        RGB(:,:,3)=uint8(zeros(size(segdna(:,:,i))));
        imshow(RGB); pause(0.1);
    end
end

%icaoberg march 26, 2012
try
 if verbose
  disp('CellOrganizer: Saving preprocessed files to disk.');
 end
 save( [ temporaryFolder ...
   filesep 'cell' num2str(cellnum) '.mat' ], 'segcell', 'segdna', 'downsample' );
catch
 if verbose
   disp('CellOrganizer: Unable to save preprocessed files to disk.');
 end
end
