function model2slml( model, documentation, file )
%MODEL2SLML Parses a generative model of protein subcellular location as an SLML instance
%model2slml( model, modelName, cellName, compartmentName, proteinName, documentation, file )
%
%Arguments          Description
%---------          -----------
%model              A structure of generative models
%documentation      A structure with the documentation (optional)
%file               Filename or file ID
%
%Example 1
%---------
%file = 'example.xml';
%documentation.name = 'Ivan E. Cao-Berg';
%documentation.email = 'icaoberg@cmu.edu';
%model2slml( model, cell, documentation, file );

% Author: Ivan E. Cao-Berg (icaoberg@cmu.edu)
% Created: May 8, 2007
% Last Update: August 27, 2010
%
% Copyright (C) 2010  Deparment of Computational Biology
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%Check input arguments
fileID = checkInputArguments( model, documentation, file );

%Write the contents of the SLML file
writeHeader( fileID );
writeDocumentation( documentation, fileID );
writeSLMLmodel( model, fileID );
writeFooter( fileID )
fclose( fileID );
end%model2slml

function fileID = checkInputArguments( model, documentation, file )
% CHECKINPUTARGUMENTS Helper method that checks the validity of the input
% arguments of the main function

%check model
if( ~isstruct( model ) )
    error('CellOrganizer: Input argument model should be of type struct');
else
    fields = fieldnames( model );
    if( length( fields ) == 3 )
        if( ~strcmpi( fields(1), 'nuclearShapeModel' ) )
            error( ['CellOrganizer: Input argument model does not contain a ' ...
                'nuclear shape model'] );
        elseif( ~strcmpi( fields(2), 'cellShapeModel' ) )
            error( ['CellOrganizer: Input argument model does not contain a ' ...
                'cell shape model'] );
        elseif( ~strcmpi( fields(3), 'proteinModel' ) )
            error( ['CellOrganizer: Input argument model should contain at ' ...
                'least one protein model'] );
        end
    elseif( length( fields ) == 4 )
        if( ~strcmpi( fields(2), 'nuclearShapeModel' ) )
            error( ['CellOrganizer: Input argument model does not contain a ' ...
                'nuclear shape model'] );
        elseif( ~strcmpi( fields(3), 'cellShapeModel' ) )
            error( ['CellOrganizer: Input argument model does not contain a ' ...
                'cell shape model'] );
        elseif( ~strcmpi( fields(4), 'proteinModel' ) )
            error( ['CellOrganizer: Input argument model should contain at ' ...
                'least one protein model'] );
        end
    end
end

%check documentation
if ~isstruct( documentation ) && ~isempty( documentation )
    error( ['CellOrganizer: Input argument documentation must be of ' ...
        'type struct'] );
end

%check file
fileID = file2fileID( file );
end%checkInputArguments